from .mainwindow import Ui_MainWindow
from .devicewidget import Ui_DeviceWidget
from .devicedetails import Ui_DeviceDetails
from .messagewidget import Ui_MessageWidget
from .messagedetails import Ui_MessageDetails
from .signalwidget import Ui_SignalWidget
from .signaldetails import Ui_SignalDetails
from .logwidget import Ui_LogWidget
from .enumwidget import Ui_EnumWidget
from .enumdetails import Ui_EnumDetails
from .logdetails import Ui_LogDetails
from .cfgwidget import Ui_CfgWidget
from .cfgdetails import Ui_CfgDetails
from .cmdwidget import Ui_CmdWidget
from .cmddetails import Ui_CmdDetails
from .cmdarg import Ui_CmdArg
from .enumvalue import Ui_EnumValue
